package models

import (
	"cvevulner/common"
	"database/sql"
	"github.com/astaxie/beego"
	"github.com/astaxie/beego/config"
	"github.com/astaxie/beego/logs"
	"github.com/astaxie/beego/orm"
	_ "github.com/go-sql-driver/mysql"
)

func Initdb() {
	BConfig, err := config.NewConfig("ini", "conf/app.conf")
	if err != nil{
		logs.Error("config init error:", err)
		return
	}
	//ConnDb()
	dbhost := BConfig.String("mysql::dbhost")
	dbport := BConfig.String("mysql::dbport")
	dbuser := BConfig.String("mysql::dbuser")
	dbname := BConfig.String("mysql::dbname")
	dbpwd := BConfig.String("mysql::dbpwd")
	//dbpwd := os.Getenv("dbpwd")
	key := beego.AppConfig.String("key")
	key1 := []byte(key)
	bytes, _ := common.DePwdCode(dbpwd, key1)
	maxidle, lerr := BConfig.Int("mysql::maxidle")
	if lerr != nil {
		maxidle = 30
	}

	maxconn, lerr := BConfig.Int("mysql::maxconn")
	if lerr != nil {
		maxconn = 3000
	}
	dns := dbuser + ":" + string(bytes) + "@tcp(" + dbhost + ":" + dbport + ")/" + dbname + "?charset=utf8"
	errx := orm.RegisterDriver("mysql", orm.DRMySQL)
	if errx != nil {
		logs.Error("RegisterDriver, orm err: ", errx)
		return
	}
	errorm := orm.RegisterDataBase("default", "mysql", dns, maxidle, maxconn)
	if errorm != nil {
		logs.Error("RegisterDataBase failed", "errorm: ", errorm)
		return
	}
	logs.Info("mysql 连接成功")
	res := CreateDb()
	if res {
		logs.Info("mysql table init success !")
	} else {
		logs.Error("mysql table init failed!")
	}
}

func ConnDb() (*sql.DB, error){
	BConfig, err := config.NewConfig("ini", "conf/app.conf")
	if err != nil{
		logs.Error("config init error:", err)
		return nil, err
	}
	dbhost := BConfig.String("mysql::dbhost")
	dbport := BConfig.String("mysql::dbport")
	dbuser := BConfig.String("mysql::dbuser")
	dbname := BConfig.String("mysql::dbname")
	dbpwd := BConfig.String("mysql::dbpwd")
	//dbpwd := os.Getenv("dbpwd")
	key := beego.AppConfig.String("key")
	key1 := []byte(key)
	bytes, _ := common.DePwdCode(dbpwd, key1)
	dns := dbuser + ":" + string(bytes) + "@tcp(" + dbhost + ":" + dbport + ")/" + dbname + "?charset=utf8"
	db, err := sql.Open("mysql", dns)
	if err != nil {
		logs.Error("连接数据库出错", err, "@tcp(" + dbhost + ":" + dbport + ")/" + dbname)
		return nil, err
	}
	logs.Info("数据库连接成功, db: ", db)
	return db, nil
}
