package models

import (
	"github.com/astaxie/beego/logs"
	"github.com/astaxie/beego/orm"
)

func GetYamlTable(gt *[]GitPackageTable) (int64, error) {
	o := orm.NewOrm()
	num, errx := o.Raw("select *"+
		" from cve_git_package_table").QueryRows(gt)
	if errx != nil {
		logs.Error("获取数据失败或者不存在, err: ", errx)
	}
	return num, errx
}

func CreateYamlTable(gt *GitPackageTable) (TableId int64, err error) {
	o := orm.NewOrm()
	var localgt GitPackageTable
	errx := o.Raw("select *"+
		" from cve_git_package_table where table_name = ? ", gt.TableName).QueryRow(&localgt)
	if errx != nil {
		// 创建
		var TableId int64
		if TableId, err = o.Insert(gt); err == nil {
			logs.Info("insert cve_git_package_table success, TableId: ", TableId, "tableName: ", gt.TableName)
		} else {
			logs.Error("insert cve_git_package_table failed, tableName:", gt.TableName, "err: ", err)
			return 0, err
		}
		return TableId, nil
	} else {
		// 更新
		gt.TableId = localgt.TableId
		if num, err := o.Update(gt); err == nil {
			logs.Info("update cve_git_package_table success, num: ", num, "TableName: ", gt.TableName)
		} else {
			logs.Error("update cve_git_package_table failed, TableName:", gt.TableName, "err: ", err)
			return 0, err
		}
		return gt.TableId, nil
	}
}

func GetSingleYaml(ge *GitOpenEuler) (bool, error){
	o := orm.NewOrm()
	if ge.Version != "" && ge.Release != "" {
		localge := GitOpenEuler{PackageName: ge.PackageName, Version:ge.Version, Release:ge.Release}
		err := o.Read(&localge, "PackageName", "Version", "Release")
		logs.Info("查询yaml数据是否存在：", ge, ", localge: ", localge, ", err: ", err)
		if err == orm.ErrNoRows {
			return false, err
		} else if err == orm.ErrMissPK {
			return false, err
		} else {
			return true, err
		}
	} else {
		localge := GitOpenEuler{PackageName: ge.PackageName}
		err := o.Read(&localge, "PackageName")
		logs.Info("查询yaml数据是否存在：", ge, ", localge: ", localge, ", err: ", err)
		if err == orm.ErrNoRows {
			return false, err
		} else if err == orm.ErrMissPK {
			return false, err
		} else {
			return true, err
		}
	}
}


func CreateYaml(ge *GitOpenEuler) (id int64, typex string, err error) {
	o := orm.NewOrm()
	var localge GitOpenEuler
	errx := o.Raw("select *"+
		" from cve_git_open_euler where package_name = ? and (version = ? or release = ?)",
		ge.PackageName, ge.Version, ge.Release).QueryRow(&localge)
	if errx != nil {
		// 创建
		logs.Info("createYaml: ", localge)
		ge.Status = 0
		var GitId int64
		if GitId, err = o.Insert(ge); err == nil {
			logs.Info("insert cve_git_open_euler success, GitId: ", GitId, "packName: ", ge.PackageName)
		} else {
			logs.Error("insert cve_git_open_euler failed, ", "packName: ", ge.PackageName, "err: ", err)
			return 0, "insert", err
		}
		return GitId, "insert", nil
	} else {
		// 更新
		logs.Info("createYaml: ", localge, "GitOpenEuler: ", ge)
		ge.Status = 1
		ge.GitId = localge.GitId
		if _, err := o.Update(ge); err == nil {
			logs.Info("update cve_git_open_euler success, GitId: ",  ge.GitId, "packName: ", ge.PackageName)
		} else {
			logs.Error("update cve_git_open_euler failed, ", "packName: ", ge.PackageName, "err: ", err)
			return ge.GitId, "update", err
		}
		return ge.GitId, "update", nil
	}
}


func CreateYamlDetail(gp *GitPackageInfo, ge GitOpenEuler) (id int64, typex string, err error) {
	o := orm.NewOrm()
	var localgp GitPackageInfo
	errx := o.Raw("select *"+
		" from cve_git_package_info where git_id = ? and package_name = ? and version = ?",
		ge.GitId, gp.PackageName, gp.Version).QueryRow(&localgp)
	if errx != nil {
		// 创建
		gp.Status = 0
		var DetailId int64
		if DetailId, err = o.Insert(gp); err == nil {
			logs.Info("insert cve_git_package_info success, DetailId: ", DetailId, ",PackageName: ", gp.PackageName)
		} else {
			logs.Error("insert cve_git_package_info failed, ", "PackageName: ", gp.PackageName, ",err: ", err)
			return 0, "insert", err
		}
		return DetailId, "insert", nil
	} else {
		// 更新
		gp.DetailId = localgp.DetailId
		if _, err := o.Update(gp); err == nil {
			logs.Info("update cve_git_package_info success, DetailId: ", gp.DetailId, ",packName: ", gp.PackageName)
		} else {
			logs.Error("update cve_git_package_info failed, ", "packName: ", gp.PackageName, ",err: ", err)
			return gp.DetailId, "update", err
		}
		return gp.DetailId, "update", nil
	}
}


func CreateYamlSubPack(gb *GitSubPack) (SubId int64, typex string, err error) {
	o := orm.NewOrm()
	var localgb GitSubPack
	errx := o.Raw("select *"+
		" from cve_git_sub_pack where detail_id = ? and ids = ? and sub_pack_name = ?",
		gb.DetailId, gb.Ids, gb.SubPackName).QueryRow(&localgb)
	if errx != nil {
		// 创建
		var SubId int64
		if SubId, err = o.Insert(gb); err == nil {
			logs.Info("insert cve_git_sub_pack success, SubId: ", SubId, ",SubPackName: ", gb.SubPackName)
		} else {
			logs.Error("insert cve_git_sub_pack failed, ", "SubPackName: ", gb.SubPackName, ",err: ", err)
			return 0, "insert", err
		}
		return SubId, "insert", nil
	}
	return localgb.SubId, "update", errx
}


func CreateYamlSubPackProvides(gs *GitSubPackProvides) (ProvideId int64, typex string, err error) {
	o := orm.NewOrm()
	var localgs GitSubPackProvides
	errx := o.Raw("select *"+
		" from cve_git_sub_pack_provides where sub_id = ? and ids = ? and provide_name = ?",
		gs.SubId, gs.Ids, gs.ProvideName).QueryRow(&localgs)
	if errx != nil {
		// 创建
		var ProvideId int64
		if ProvideId, err = o.Insert(gs); err == nil {
			logs.Info("insert cve_git_sub_pack_provides success, ProvideId: ", ProvideId, "ProvideName: ", gs.ProvideName)
		} else {
			logs.Error("insert cve_git_sub_pack_provides failed, ", "ProvideName: ", gs.ProvideName, "err: ", err)
			return 0, "insert", err
		}
		return ProvideId, "insert", nil
	}
	return localgs.ProvideId, "update", errx
}


func CreateYamlSubPackRequiredb(gs *GitSubPackRequiredby) (Id int64, typex string, err error) {
	o := orm.NewOrm()
	var localgs GitSubPackRequiredby
	errx := o.Raw("select *"+
		" from cve_git_sub_pack_requiredby where provide_id = ? and requiredb = ?",
		gs.ProvideId, gs.Requiredby).QueryRow(&localgs)
	if errx != nil {
		// 创建
		var Id int64
		if Id, err = o.Insert(gs); err == nil {
			logs.Info("insert cve_git_sub_pack_requiredby success, Id: ", Id, "requiredb: ", gs.Requiredby)
		} else {
			logs.Error("insert cve_git_sub_pack_requiredby failed, ", "requiredb: ", gs.Requiredby, "err: ", err)
			return 0, "insert", err
		}
		return Id, "insert", nil
	}
	return localgs.Id, "update", errx
}


func CreateYamlSubPackRequires(gs *GitSubPackRequire) (RequireId int64, typex string, err error) {
	o := orm.NewOrm()
	var localgs GitSubPackRequire
	errx := o.Raw("select *"+
		" from cve_git_sub_pack_require where sub_id = ? and ids = ? and require_name = ?",
		gs.SubId, gs.Ids, gs.RequireName).QueryRow(&localgs)
	if errx != nil {
		// 创建
		var RequireId int64
		if RequireId, err = o.Insert(gs); err == nil {
			logs.Info("insert cve_git_sub_pack_require success, RequireId: ", RequireId, "RequireName: ", gs.RequireName)
		} else {
			logs.Error("insert cve_git_sub_pack_require failed, ", "RequireName: ", gs.RequireName, "err: ", err)
			return 0, "insert", err
		}
		return RequireId, "insert", nil
	}
	return localgs.RequireId, "update", errx
}