package taskhandler

import (
	"cvevulner/util"
	"encoding/json"
	"errors"
	"github.com/astaxie/beego/logs"
	"os"
	"strings"
)


func GetOautToken(gt GiteeToken) {
	url := "https://gitee.com/oauth/token"
	var req util.RequestInfo
	req.Url = url
	req.Data = make(map[string]string)
	req.Data["grant_type"] = gt.GrantType
	req.Data["username"] = gt.UserName
	req.Data["password"] = gt.Password
	req.Data["client_id"] = gt.ClientId
	req.Data["client_secret"] = gt.ClientSecret
	req.Data["scope"] = gt.Scope
	resp, err := util.PostUrlEncoded(req)
	if err != nil {
		logs.Error("获取 token 失败，url: ", url, "请求参数：", gt, "err:", err)
		return
	}
	var respDict map[string]interface{}
	err =json.Unmarshal(resp, &respDict)
	if err != nil {
		logs.Error(err)
		return
	}
	if _, ok := respDict["access_token"]; !ok {
		logs.Error("获取token失败, err: ", ok, "url: ", url)
		return
	}
	GitToken := respDict["access_token"].(string)
	os.Setenv("GITEE_TOKEN", GitToken)
}


func GetCollaboratorInfo(accessToken string, owner string, path string) (string, error){
	if accessToken != "" && owner != "" && path !="" {
		url := "https://gitee.com/api/v5/repos/" + owner + "/" + path + "/collaborators?access_token=" + accessToken
		collabor, err:= util.HttpGet(url)
		if err == nil && collabor != nil {
			for _, value := range collabor {
				if _, ok := value["id"]; !ok {
					logs.Error("collaborators, err: ", ok, "url: ", url)
					continue
				}
				assignee := ""
				flag := false
				for mapKey, mapValue := range value {
					if mapKey == "login" {
						assignee = mapValue.(string)
					}
					if mapKey == "permissions" {
						mapValuex := mapValue.(map[string]interface{})
						for perKey, perValue := range mapValuex {
							if perKey == "admin" && perValue.(bool) == true{
								flag = true
							}
						}
					}
				}
				if assignee != "" && flag {
					return assignee, nil
				}
			}
		} else {
			logs.Error("获取仓库责任人失败, owner:", owner, "path:", path, "err: ", err)
			return "", err
		}
	}
	return "", errors.New("参数错误")
}

func GetBranchesInfo(accessToken string, owner string, path string) (string, error){
	branchName := ""
	if accessToken != "" && owner != "" && path != "" {
		url := "https://gitee.com/api/v5/repos/"+ owner +"/"+ path +"/branches?access_token=" + accessToken
		branch, err:= util.HttpGet(url)
		if err == nil && branch != nil {
			for _, value := range(branch) {
				if _, ok := value["name"]; !ok {
					logs.Error("branches, err: ", ok, "url: ", url)
					continue
				}
				mapValue := value["name"].(string)
				if mapValue != "" && len(mapValue) > 3 {
					subStr := mapValue[len(mapValue) - 3:]
					if strings.ToUpper(subStr) == "LTS" {
						branchName = mapValue
						break
					}
				}
			}
		} else {
			logs.Error("获取分支名称失败， err: ", err, "owner: ", owner, "path: ", path)
			return branchName, err
		}
	}
	return branchName, nil
}
